require 'thor'
require 'active_support/core_ext/string/inflections'

require_relative 'monkey/config.rb'

module Monkey

  # def self.init_win_service
  #   require 'win32/service'
  #   include Win32
  # end

  # def self.init_win_task_scheduler
  #   require 'win32/taskscheduler'
  #   include Win32
  # end

  class CLI < Thor
    include Thor::Actions

    def self.exit_on_failure?; true; end

    desc 'see [DIRECTORY]', 'Register DIRECTORY as a task suite'
    def see(dir='')
      full_dir = File.expand_path(dir)

      config = Config.new

      if config.empty?
        config['task_suites'] = [ full_dir ]
      else
        task_suites = config['task_suites']
        exit if task_suites.include? full_dir
        task_suites << full_dir
      end

      config.save!
    end

    desc 'unsee [DIRECTORY]', 'Deregister DIRECTORY as a task suite'
    def unsee(dir='')
      full_dir = File.expand_path(dir)

      config = Config.new

      unless config.empty?
        task_suites = config['task_suites']
        task_suites.delete full_dir
      end

      config.save!
    end

    desc 'do [TASKS]', 'Run TASKS in sequence'
    def do(*tasks)
      require_relative 'monkey/clock.rb'

      tasks = Task.tasks if tasks.empty?

      tasks.each do |task|
        Monkey.const_get(task.to_s.camelize).new
      end
    end

    desc 'start', 'Start Monkey process, running tasks when triggered'
    def start
      current_dir = File.expand_path(File.dirname(__FILE__))
      Dir.chdir current_dir
      exec("clockwork monkey/clock.rb")
    end

    # desc 'startup', 'Setup Monkey to run at startup.'
    # def startup
    #   require_relative 'helpers/computer.rb'

    #   abort 'This is only supported on Windows for now.' unless Computer.windows?
    #   Monkey.init_win_task_scheduler
    #   current_dir = File.expand_path(File.dirname(__FILE__))

    #   #username = ask('What user account would you like to use to run Monkey?')
    #   #domain = ask('What domain is the user account in?')

    #   ts = Monkey::TaskScheduler.new
    #   now = Time.now
    #   trigger = {
    #     start_year:   now.year,
    #     start_month:  now.month,
    #     start_day:    now.day,
    #     start_hour:   now.hour,
    #     start_minute: now.min,
    #     trigger_type: Monkey::TaskScheduler::TASK_EVENT_TRIGGER_AT_SYSTEMSTART
    #   }
    #   ts.new_work_item('Monkey', trigger)
    #   ts.application_name = "#{`where ruby`.chomp}"
    #   ts.parameters = "-C #{current_dir} monkey.rb start"
    #   #ts.set_machine(Socket.gethostname, username, domain)
    #   p ts.set_account_information('Moo', 'test')
    #   ts.activate('Monkey')
    # end

    # desc 'bg', 'Run Monkey as a daemon or Windows service, configured to run automatically at start up'
    # def bg
    #   require_relative 'helpers/computer.rb'

    #   abort 'This is only supported on Windows for now.' unless Computer.windows?
    #   Monkey.init_win_service
    #   current_dir = File.expand_path(File.dirname(__FILE__))
    #   monkey_dir = File.join(current_dir, 'monkey')

    #   Monkey::Service.create({
    #     service_name: 'Monkey',
    #     host: nil,
    #     service_type: Monkey::Service::WIN32_OWN_PROCESS,
    #     description: 'Monkey is an IT automation and scripting framework for Ruby.',
    #     start_type: Monkey::Service::AUTO_START,
    #     error_control: Monkey::Service::ERROR_NORMAL,
    #     binary_path_name: "#{`where ruby`.chomp} -C #{monkey_dir} service.rb",
    #     load_order_group: 'Network',
    #     dependencies: nil,
    #     display_name: 'Monkey'
    #   })

    #   Monkey::Service.start 'Monkey'
    # end

    # desc 'bgpause', 'Stop the Monkey daemon until it is started again'
    # def bgpause
      
    # end

    # desc 'bgstop', 'Deregister the daemon or Windows service'
    # def bgstop
    #   require_relative 'helpers/computer.rb'
    #   abort 'This is only supported on Windows for now.' unless Computer.windows?
    #   Monkey.init_win_service
    #   `sc stop testservice`
    #   Monkey::Service.delete 'Monkey'
    # end

  end

end

Monkey::CLI.start(ARGV)